#!/usr/bin/perl

# 0 - motif id file
# 1 - predictors: go, prior, predictor motifs=bs
# 2 - architecture file
# 3 - go tree file - not numbered truepath, but tree
# 4 - test set <unid> <gos>
# 5 - go depth file
# 6 - mode: if set to -1, use first column as UniProt ID to get architecture from file above
#		if set to a column number, use architecture (as PF00008*PF00007....) in that column

# first read all motifs

# open motif file

%motif2id = ();
%id2motif = ();

open (FH, $ARGV [0]);

while (<FH>) {

 $aLine = $_;
 chomp ($aLine);

 @words = split (" ", $aLine);
 $id = $words [0];
 $motif = $words [1];

 $motif2id {$motif} = $id;
 $id2motif {$id} = $motif;

}

close (FH);

# depth file

%depths = ();

open (FH, $ARGV [5]);

while (<FH>) {

 $aLine = $_;
 chomp ($aLine);

 @words = split (" ", $aLine);
 $go = $words [0];
 $depth = $words [1];

 $depths {$go} = depth;

}

close (FH);

# then read the profiles

%go2id = ();
%id2go = ();
$goCount = 0;
%preds = ();
%priors = ();

open (FH, $ARGV [1]);

while (<FH>) {

 $aLine = $_;
 chomp ($aLine);

 @words = split (" ", $aLine);

 $goTerm = $words [0];

 if (! exists ($go2id {$goTerm})) {

  $go2id {$goTerm} = $goCount;
  $id2go {$goCount} = $goTerm;
  $goCount++;

 }

 $prior = $words [1];

 $priors {$go2id {$goTerm}} = $prior; 

 $noMotifs = @words;

 for ($i = 2; $i < $noMotifs; $i++) {

  $words [$i] =~ /^([^=]*)=([^=]*)$/;
  $aMotif = $1;
  $aFBS = $2;

  if (exists ($preds {$aMotif})) {

   $preds {$aMotif} = $preds {$aMotif}." ".$go2id {$goTerm}."=".$aFBS;

  }

  else {

   $preds {$aMotif} = $go2id {$goTerm}."=".$aFBS;

  }

 }

}

close (FH);

# domain architectures

%arch = ();

open (FH, $ARGV [2]);

while (<FH>) {

 $aLine = $_;
 chomp ($aLine);

 # this is the architecture file...

 @words = split (" ", $aLine);
 $id = $words [0];
 $anArch = $words [2];
 $arch {$id} = $anArch;

}

close (FH);

# moreover, we also need the truepaths!

%completions = ();

open (FH, $ARGV [3]);

while (<FH>) {

 $aLine = $_;
 chomp ($aLine);

 @words = split (" ", $aLine);

 $sS = "";

 foreach $aGo (@words) {

  if (! exists ($go2id {$aGo})) {

   $go2id {$aGo} = $goCount;
   $id2go {$goCount} = $aGo;
   $goCount++;

  }

  $sS = $sS.$go2id {$aGo}." ";

 }

 chomp ($sS);

 $completions {$go2id {$words [0]}} = $sS;

}

close (FH);

#

# then read test set

open (FH, $ARGV [4]);

while (<FH>) {

 $aLine = $_;
 chomp ($aLine);

 %lPreds = (); # what is predicted here?
 %lFBS = ();
 %mFBS = (); # max support
 %cFBS = ();

 # this file is the main dataset, of type <id> <gos>

 @words = split (" ", $aLine);

 $N = @words;

 if ($ARGV [6] == - 1) {

  $id = $words [0];
  $anArch = $arch {$id};

 }

 else {

  $id = $words [0];
  $anArch = $words [$ARGV [6]];

 }

 # first make it a set

 @aDs = split (/\*/, $anArch);
 %lDs = ();

 foreach $aD (@aDs) {

  if (! exists ($lDs {$aD})) {

   $lDs {$aD} = "";

  }

 }

 # then order it

 @sDs = sort { lc ($a) cmp lc ($b) } keys % lDs;
 $Nd = @sDs;

 # motifs in this example:

 $mS = "";

 # then take all the permutations

 for ($i = 1; $i < exp (log (2) * $Nd); $i++) {

  @posS = split (//, dec2bin ($i, $Nd));

  $tS = "";

  for ($j = 0; $j < $Nd; $j++) {

   if ($posS [$j] == 1) {

    $tS = $tS.$sDs [$j]."*";

   }

  }

  chop ($tS);
 
  # tS is a motif!
  # one of those present
  # and a source of potentially predictive power

  foreach $alP (split (" ", $preds {$motif2id {$tS}})) {

   # more

   $alP =~ /^([^=]*)=([^=]*)$/;
   $goId = $1;
   $FBS = $2; 

   if (exists ($lFBS {$goId})) {

    $lFBS {$goId} = $lFBS {$goId} + $FBS;
    $cFBS {$goId}++;

    if ($FBS > $mFBS {$goId}) {

     $mFBS {$goId} = $FBS;

    }

   }

   else {

    $lFBS {$goId} = $FBS;
    $mFBS {$goId} = $FBS;
    $cFBS {$goId} = 1;

   }

  }

 }

 # so?

 foreach $goId (keys % lFBS) {

  # NOTE this uses AVERAGE contribution!

#  if ($mFBS {$goId} >= $priors {$goId}) {
  if ($lFBS {$goId} / $cFBS {$goId} >= $priors {$goId}) {

   # and truepath...

   foreach $aGo (split (" ", $completions {$goId})) {

    $lPreds {$aGo} = "";

   }

  }

 }

 # so, for this row, we predict keys % lPreds

 $sS = "";

 foreach $anID (sort { lc($a) cmp lc($b) } keys % lPreds) {

  $sS = $sS." ".$id2go {$anID};

 }

 print "$id $anArch$sS\n";

}

close (FH);

sub dec2bin {

    my $str = unpack("B32", pack("N", shift));
    my $nos = shift;
    $str = substr ($str, 32 - $nos, $nos);
    return $str;

}

sub max {

    if ($_[0]<$_[1]) {return $_[1]} else {return $_[0]};

} 